#!/usr/bin/env python

import os

def pfxlist_generate_juniper (pfx, pfxname, pfxlen):
	result = []
	result.append("policy-options {")
	result.append("	replace: policy-statement %s {" % pfxname)
	result.append("		term prefixes {")
	result.append("			from {")

	for line in pfx():
		line = line.strip()
		if line == '':
			continue
		result.append("				route-filter %s upto /%s;" % (line, pfxlen))

	result.append("			}")
	result.append("			then next policy;")
	result.append("		}")
	result.append("		then reject;")
	result.append("	}")
	result.append("}")
	return result

def pfxlist_generate_juniper_xml (pfx, pfxname, pfxlen):
	result = []
	result.append("<policy-options>")
	result.append("	<policy-statement operation=\"replace\">")
	result.append("		<name>%s</name>" % pfxname)
	result.append("		<term>")
	result.append("			<name>prefixes</name>")
	result.append("			<from>")

	for line in pfx():
		line = line.strip()
		if line == '':
			continue
		result.append("				<route-filter>")
		result.append("					<address>%s</address>" %line)
		result.append("					<upto>%s</upto>" % pfxlen)
		result.append("				</route-filter>")

	result.append("			</from>")
	result.append("			<then>")
	result.append("				<next>policy</next>")
	result.append("			</then>")
	result.append("		</term>")
	result.append("		<then>")
	result.append("			reject")
	result.append("		</then>")
	result.append("	<policy-statement>")
	result.append("</policy-options>")
	return result

def pfxlist_generate_cisco (pfx, pfxname, pfxlen):
	result = []
	result.append("conf t")
	result.append("no ip prefix-list %s" % pfxname)
	for line in pfx():
		line = line.strip()
		if line == '':
			continue
		
		le = ""
		prefix = line.split('/')
		if int(prefix[1]) < int(pfxlen):
			le = "le %s" % pfxlen
		result.append("ip prefix-list %s permit %s/%s %s" % (pfxname, prefix[0],prefix[1],le))
	
	result.append("end")
	result.append("write mem")
	return result

def pfxlist_generate_extreme (pfx, pfxname, pfxlen):
	result = []
	pfxname.replace(':','-')
	
	result.append("delete access-profile %s" % pfxname)
	result.append("create access-profile %s type ipaddress" % pfxname)
	
	counter = 10

	for line in pfx():
		line = line.strip()
		if line == '':
			continue
		
		prefix = line.split('/')
		le =""
		if int(prefix[1]) >= int(pfxlen):
			le = "exact"
		
		result.append("configure access-profile %s add %d permit ipaddress %s/%s %s" % (pfxname, counter, prefix[0], prefix[1],le))
		counter += 10
		
	
	result.append("save")
	return result

def pfxlist_generate (format,asn,pfxstr,pfxlen,db,suffix):
	f = os.path.join(db,asn+suffix)
	
	if not os.path.exists(f) or not os.access(f,os.R_OK):
		print >> sys.stderr, "Cannot access IRRDB prefix list file %s" % f
		return False

	try:
		pfxfile = open(f,'r')
	except:
		print >> sys.stderr, "Cannot open IRRDB prefix list file %s" %f
		return False
	
	run = {}
	run['cisco'] = pfxlist_generate_cisco
	run['extreme'] = pfxlist_generate_extreme
	run['juniper'] = pfxlist_generate_juniper
	run['juniper-xml'] = pfxlist_generate_juniper_xml
	
	pfxname = pfxstr % int(asn)
	try:
		result = run[format](pfxfile.readlines,pfxname,pfxlen)
	except:
		return None
		
	pfxfile.close()
	return result

if __name__ == '__main__':
	import os
	import sys
	parent = os.path.abspath(os.path.join(os.path.abspath(os.path.dirname(__file__)),'..'))
	sys.path.append(parent)
	from conf.param import option
	from conf.irrpt import cfg
	
	format = cfg['pfxgen']['default_format']
	pfxstr = cfg['pfxgen']['default_pfxstr']
	pfxlen = cfg['pfxgen']['default_pfxlen']
	db = cfg['paths']['db']
	suffix = cfg['aggregate']['suffix']
	cmdline = option.get('cmdline',[])
	for asn in cmdline:
		r = pfxlist_generate (format,asn,pfxstr,pfxlen,db,suffix)
		if r is None:
			print >> sys.stderr, "Cannot read IRRDB prefix list for asn %s" % asn
			sys.exit(1)
		
		for l in r:
			print l
