#!/usr/bin/env python

# XXX: Todo : handle missing AS-MACRO with "<AS-UNSPECIFIED>"

import time
import os
import sys

def parse (stdin,display,prefs = {'peer': 50, 'transit': 0, 'customer' : 100}):
	data = {}
	
	for line in stdin:
		try:
			line = line.strip()
			router,atype,group,asn,announce,src,dest,macro,mail,name = line.split('|')
		except:
			print >> sys.stderr, 'wrong data for ripe parser :', line
			sys.exit(1)

		if not data.has_key(atype):
			data[atype] = {}
		if not data[atype].has_key(group):
			data[atype][group] = []
		data[atype][group].append((asn,announce,src,dest,macro,mail,name))
	
	result = []
	for atype in data.keys():
		if not atype in display:
			continue
		
		# what peer / transit are we printing
		c_group = ''
		# all peers have the same local-pref
		pref = prefs[atype]

		for group in data[atype]:
			if group != c_group:
				c_group = group
				result.append("remarks:        %s" % group.upper().replace('-',' '))
				result.append("remarks:        ===================================================")

			ordered = {}
			for asn,announce,src,dest,macro,mail,name in data[atype][group]:
				a,b,c,d = dest.split('.')
				ipn = (int(a) << 24) + (int(b) << 16) + (int(c) << 8) + int(d)
				ordered[ipn] = asn,announce,src,macro,mail,name

			keys = ordered.keys()
			keys.sort()

			for ipn in keys:
				asn,announce,src,macro,mail,name = ordered[ipn]

				d = ipn & 255 
				ipn = ipn >> 8
				c = ipn & 255
				ipn = ipn >> 8
				b = ipn & 255
				a = ipn >> 8
				dest = "%d.%d.%d.%d" % (a,b,c,d)

				if macro == 'ANY': macro += ' and not {0.0.0.0/0}'
				result.append("remarks:        %s" % name)
				if pref != None:
					result.append("import:         from AS%-7s %-16s at %-16s action pref=%-3s; accept %s;" % (asn,dest,src,pref,macro))
				else:
					result.append("import:         from AS%-7s %-16s at %-16s accept %s;" % (asn,dest,src,macro))
				result.append("export:         to   AS%-7s %-16s at %-16s announce %s;" % (asn,dest,src,announce))
				result.append("remarks:        ---------------------------------------------------")
	return result

if __name__ == '__main__':
	parent = os.path.abspath(os.path.join(os.path.abspath(os.path.dirname(__file__)),'..'))
	sys.path.append(parent)
	from conf.param import option
	from conf.ripe import parse as parse_ripe
	
	def usage (problem):
		print >> sys.stderr , 'usage: %s [files]' % sys.argv[0]
		print >> sys.stderr , problem
		sys.exit(1)

	display_options = ['transit','peer','customer']
	display = []
	for d in display_options:
		if option.has_key(d):
			display.append(d)
	if display == []:
		display = display_options
	
	padding = parse_ripe()
	body = parse(sys.stdin,display)

	if not padding.has_key('header') or not padding.has_key('footer'):
		usage('no configuration file found')

	replace = {'generated':time.strftime("%Y%m%d %H:%M:%S")}

	print '\n'.join(padding['header']) % replace
	print '\n'.join(body)
	print '\n'.join(padding['footer']) % replace
	
	sys.exit(0)

