#!/usr/bin/env python

import re

def parse (stream,our_asn,our_peering_policy,peer,transit,customer):
	our_peering_policy = our_peering_policy.upper()
	
	if peer == []:
		peer = ['something-unique-the-regex-will-not-match-for-peer']
	if customer == []:
		customer = ['something-unique-the-regex-will-not-match-for-customer']
	if transit == []:
		transit = ['something-unique-the-regex-will-not-match-for-transit']
	
	expr_router = re.compile('\s+host-name (.*);')
	expr_group = re.compile('\s+(inactive:\s*)?group (.*) {')
	expr_neig = re.compile('\s+(inactive:\s*)?neighbor (.*) {')
	expr_desc = re.compile('\s+description "(.*?)\|(.*?)\|([^|]*)\|?(.*)"')
	expr_asn = re.compile('\s+peer-as (.*);')
	expr_local = re.compile('\s+local-address (.*);')
	expr_export = re.compile('\s+export ')
	expr_peer = re.compile('\s+export \[ *.* *('+'|'.join(peer)+') *.* *]?')
	expr_transit = re.compile('\s+export \[\s*.*\s*('+'|'.join(transit)+')\s*.*\s*]?')
	expr_customer = re.compile('\s+export \[\s*.*\s*('+'|'.join(customer)+')\s*.*\s*]?')

	# Multiple peering with the same ASN will generate multiple lines

	skip = True
	mail = ''
	src = ''
	dest = ''
	macro = ''
	name = ''
	announce = ''
	group = ''
	type = 'unknown'
	router = ''
	inactive_group = False
	inactive_neighbor = False

	for line in stream:
		line = line[:-1]
		# we need to pass the group definition as we do not want to parse it
		if skip:
			if line == 'system {':
				skip = False
			if line.endswith('  bgp {'):
				skip = False
			if line == '}':
				skip = True
			continue
		
		# and as we have the test, we are leaving the logical router or the protocol section
		if line[0] == '}':
			skip = True
			continue
	
		match = expr_router.search(line)
		if match:
			router = match.group(1)
	
		# This may not work will all configuration
		match = expr_group.search(line)
		if match:
			inactive_neighbor = False
			group = match.group(2)
			if match.group(1) == None:
				inactive_group = False
			else:
				inactive_group = True
		
		if inactive_group:
			continue
		
		# set the destination IP of the BGP session
		match = expr_neig.search(line)
		if match:
			dest = match.group(2)
			if match.group(1) == None:
				inactive_neighbor = False
			else:
				inactive_neighbor = True
		
		if inactive_neighbor:
			continue
		
		# set the local IP of the BGP session
		match = expr_local.search(line)
		if match:
			src = match.group(1)

		# Peering or Transit
		match = expr_export.search(line)
		match_count = 0
		if match:
			match_peer = expr_peer.search(line)
			match_transit = expr_transit.search(line)
			match_customer = expr_customer.search(line)
			if match_peer:
				type = 'peer'
				match_count += 1
			if match_transit:
				type = 'transit'
				match_count += 1
			if match_customer:
				type = 'customer'
				match_count += 1
			if match_count != 1:
				type = 'unknown'
				continue

		# Parse the information stored in the description
		match = expr_desc.search(line)
		if match:
			macro,name = match.group(1).strip(), match.group(2).strip()
			mail,announce = match.group(3).strip(), match.group(4).strip()
			#print "[%s][%s][%s][%s]" % (macro,name,mail,announce)
			#print line
		else:
			#if line.find('description') > 0 :
			#	print line[:-1]
			announce = ''


		match = expr_asn.search(line)
		if match:
			macro = macro.upper()
			announce = announce.upper()

			asn = match.group(1)
			if type == 'peer' and macro != '' and dest != '' and src != '':
				if announce == '': announce = our_peering_policy
				export = type
			elif type == 'transit' and dest != '' and src != '' and macro == 'ANY':
				if announce == '': announce = 'ANY'
				export = type
			elif type == 'customer' and dest != '' and src != '' and macro != '':
				if announce == '': announce = 'ANY'
				export = type
			elif asn == our_asn:
				type = 'ibgp'
				export = type
			elif type == 'unknown':
				export = type
			else:
				export = 'error ' + type

			if router == '':
				print >> sys.stderr, 'can not extract router name'
				sys.exit(1)

			yield "%s|%s|%s|%s|%s|%s|%s|%s|%s|%s" % (router,export,group,asn,announce,src,dest,macro,mail,name)
			# do not clear src or group, it could be define for more than one peer
			dest = ''
			macro = ''
			name = ''
			announce = ''
			mail = ''
			macro = ''
			dest = ''

if __name__ == '__main__':
	import os
	import sys
	parent = os.path.abspath(os.path.join(os.path.abspath(os.path.dirname(__file__)),'..'))
	sys.path.append(parent)
	from conf.param import option
	
	def usage (problem):
		print >> sys.stderr , ''
		print >> sys.stderr , 'usage: %s asn [options] asn object' % sys.argv[0]
		print >> sys.stderr , ''
		print >> sys.stderr , 'parse one or more juniper configuration provided on stdin'
		print >> sys.stderr , 'and generate an easily parseable list of all bgp session'
		print >> sys.stderr , ''
		print >> sys.stderr , 'asn    : the asn of your network'
		print >> sys.stderr , 'object : your peering object'
		print >> sys.stderr , ''
		print >> sys.stderr , ' -p, --peer     export-statement  , should an import or export contain export-statement, classify it as peer'
		print >> sys.stderr , ' -c, --customer export-statement  , should an import or export contain export-statement, classify it as customer'
		print >> sys.stderr , ' -t, --transit  export-statement  , should an import or export contain export-statement, classify it as tranist'
		print >> sys.stderr , 'multiple use of the same option is possible'
		print >> sys.stderr , ''
		print >> sys.stderr , problem
		sys.exit(1)

	try:
		our_asn = option['cmdline'][0]
		if our_asn[:2] == 'AS':
			our_asn = our_asn[:2]
		our_asn = str(int(our_asn))
	except:
		usage('wrong or missing ASN')

	try:
		our_peering_policy = option['cmdline'][1]
	except:
		usage('wrong or missing AS-Macro')

	peer = option.get('peer',[])
	customer = option.get('customer',[])
	transit = option.get('transit',[])

	if peer == True: peer = []
	if transit == True: transit = []
	if customer == True: customer = []
	if peer == [] and customer == [] and transit == []:
		usage('you need to specify at least on of the option -p,-t,-c with a parameter')
	
	found = False
	for r in parse(sys.stdin,our_asn,our_peering_policy,peer,transit,customer):
		found = True
		print r
	
	if not found:
		print >> sys.stderr, 'could not find a single bgp relation'
		sys.exit(1)
