#!/usr/bin/env python

# ./app/import/juniper.py ftpd -d | ./app/parse/juniper.py 30740 AS-EXA -p export-peer -t export-transit -c export-customer | ./app/export/ripe.py

import time
import os
import sys

from source.juniper import locate as locate_juniper
from parse.juniper import parse as parse_juniper
from export.ripe import parse as parse_ripe

def usage (problem):
	print >> sys.stderr , ''
	print >> sys.stderr , 'usage: %s [options] asn asn object' % sys.argv[0]
	print >> sys.stderr , ''
	print >> sys.stderr , 'parse one or more juniper configuration provided on stdin'
	print >> sys.stderr , 'and generate an easily parseable list of all bgp session'
	print >> sys.stderr , ''
	print >> sys.stderr , 'asn    : the asn of your network'
	print >> sys.stderr , 'object : your peering object'
	print >> sys.stderr , ''
	print >> sys.stderr , ' -f  --folder   directory         , where are the configuration stored'
	print >> sys.stderr , ' -p, --peer     export-statement  , should an import or export contain export-statement, classify it as peer'
	print >> sys.stderr , ' -c, --customer export-statement  , should an import or export contain export-statement, classify it as customer'
	print >> sys.stderr , ' -t, --transit  export-statement  , should an import or export contain export-statement, classify it as tranist'
	print >> sys.stderr , 'multiple use of the same option is possible'
	print >> sys.stderr , ''
	print >> sys.stderr , problem
	sys.exit(1)

def dump_routers (routers):
	for r in routers:
		try:
			f = open(r,'r')
			for line in f:
				yield line
			f.close()
		except:
			print >> sys.stderr, 'could not display the content of %s' % r
			sys.exit(1)

def iters (members):
	for i in members:
		for e in i: yield e

if __name__ == '__main__':
	from conf.param import option
	from conf.irrpt import cfg
	from conf.ripe import secret, parse as parse_ripe_conf
	from tools.mail import SendMail
	
	parent = os.path.abspath(os.path.abspath(os.path.dirname(__file__)))
	sys.path.append(parent)

	display_options = ['transit','peer','customer']	

	config_type = None
	our_asn = None
	our_peering_policy = None
	directory = None
	export = {}
	export['peer'] = []
	export['customer'] = []
	export['transit'] = []
	
	try:
		config_type = cfg['global']['parse']
		our_asn = cfg['global']['asn']
		our_peering_policy = cfg['global']['as-macro']
		directory_juniper = cfg['paths']['juniper']
		for name in cfg['irrdb']['export'].split(' '):
			if cfg[config_type].has_key(name):
				export[name] = cfg[config_type][name].split(' ')
	except:
		usage('invalid irrpt.conf file')

	try: our_asn = option['cmdline'][0]
	except: pass
	
	try: our_peering_policy = option['cmdline'][1]
	except: pass

	try : directory = option.get['folder'][0]
	except: pass

	try:
		if our_asn[:2] == 'AS':
			our_asn = our_asn[:2]
		our_asn = str(int(our_asn))
	except:
			usage('wrong or missing ASN')

	if our_peering_policy == None:
		usage('wrong or missing AS-Macro')

	peer = option.get('peer',export.get('peer',[]))
	customer = option.get('customer',export.get('customer',[]))
	transit = option.get('transit',export.get('transit',[]))
	
	if peer == [] and customer == [] and transit == []:
		usage('you need to specify at least on of the option -p,-t,-c with a parameter')

	display = []
	for d in display_options:
		if option.has_key(d):
			display.append(d)
	
	if display == []:
		display = display_options
	
	padding = parse_ripe_conf()
	if not padding.has_key('header') or not padding.has_key('footer'):
		usage('no configuration file found')

	if config_type == 'juniper':
		parse = parse_juniper
		locate = locate_juniper
		directory = directory_juniper
	else:
		usage('unimplemted, you can help !')
	
	files = locate(directory)
	
	r = []
	opened = []
	try:
		for n in files:
			f = open(n)
			opened.append(f)
			iterator = parse(f,our_asn,our_peering_policy,peer,transit,customer)
			r.append(iterator)
		body = parse_ripe(iters(r),display)
	except:
		print >> sys.stderr, 'could not display the content of %s' % n
		sys.exit(1)
		# The files will be closed by the os
	
	for f in opened:
		f.close()
	
	replace = {'generated':time.strftime("%Y%m%d %H:%M:%S")}

	message = ''
	message += '\n'.join(padding['header']) % replace
	message += '\n'
	message += '\n'.join(body)
	message += '\n'
	message += '\n'.join(padding['footer']) % replace

	password = secret()
	message += '\npassword:       %s' % password
	
	sent = False
	if password:
		mail = SendMail()
		try:
			sender = cfg['update']['from']
		except:
			print >> sys.stderr , 'could not send ripe update, not sender defined'
			sys.exit(1)
		
		# XXX: Should really perform an DNS MX QUERY here ..
		mail.mx('postman.ripe.net')
		mail.subject('')
		mail.sender(sender)
		mail.recipient('auto-dbm@ripe.net')
		sent = mail.send(message)
		
		if not sent:
			print >> sys.stderr , 'could not send ripe update',
			if mail.error != None:
				print >> sys.stderr , str(mail.error)
			else:
				print >> sys.stderr
			sys.exit(1)
	else:
		print message
	
	sys.exit(0)
