#!/usr/bin/env python

import sys
import socket
import time

class IRR:
	def __init__ (self,host,port,fp=None):
		self.host = host
		self.port = port
		
		self.caching = False
		self.cache = {}
		self.debug = 0
		self.lastcommand = ''
		self.buf = ""
		
		self.fp = fp
		if fp == None:
			self.connection(host,port)
	
	def __delete__ (self):
		self.close()
	
	def connection (self,host,port):
		self.host = host
		
		try:
			self.port = int(port)
			self.fp = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
			self.fp.connect((host,self.port))
		except:
			self.notice('Could not connnect to IRRd server')
			raise
		
		self._multi_mode()
		self._identify()

		return self

	def close(self):
		if self.fp != None:
			self._send("!q\n")
			self.fp.close()
		self.fp = None

	def _multi_mode (self):
		return self._send('!!\n')
	
	def _identify (self):
		self._send("!nIRRDummyTools\n")
		r = self._response()
		
		if r == True:
			return True
		else:
			self.notice('the whois server speaks a foreign language')
			return False
	
	def _send(self,output):
		self.lastcommand = output.strip()
		try:
			self.notice("sending : %s" % output.strip(),2)
			sent = self.fp.send(output)
			if sent == 0:
				raise RuntimeError, "socket connection broken"
			return True
		except:
			self.notice("unexpected error whist writing on socket.")
			return False
	
	def _readline (self,size=0):
		try:
			if self.buf != "":
				chunk = self.buf
				self.buf = ""
			else:
				chunk = ""
			
			count = chunk.find('\n')
			
			# This takes too much CPU
			#self.fp.setblocking(0)
			self.fp.setblocking(1)
			self.fp.settimeout(0.05)
			while count < 0:
				try:
					while True:
						r = self.fp.recv(1024)
						chunk += r
				except:
					#if self.debug: notce('waiting for data from network')
					# using the count function of the string create a bug !!!
					count = chunk.find('\n')
			
			if count >= 0:
				sep = chunk.find('\n')
				self.buf = chunk[sep+1:]
				self.notice('buffered %d byte(s)' % len(self.buf),3)
				self.notice('buffered %s' % self.buf.replace('\n','\\n'),4)
				chunk = chunk[:sep+1]
			self.notice('returning %d byte(s)' % len(chunk),3)
			self.notice('returning %s' % chunk.replace('\n','\\n'),4)
			
			return chunk
		
		except:
			self.notice("unexpected read() error on socket.")
			raise
	
	def _response (self):
		try:
			line = self._readline()
			header = line[0]
		except:
			return False
		
		if header == 'C':
			return True
		if header == 'D':
			self.notice("key not found - query: %s" % self.lastcommand,1)
			return False
		if header == 'E':
			return True
		if header == 'F':
			self.notice("query %s failed: %s" % (self.lastcommand,header[1:]),1)
			return False
		if header == '%':
			self.notice("not entries found - query: %s" % self.lastcommand,1)
			return False
		
		datalen = 0
		try:
			if line[0] == 'A':
				datalen = int(line[1:].strip())
			else:
				raise
		except:
			self.notice("parse error looking for data length - query: %s" % self.lastcommand,1)
			return False
		
		data = ""
		try:
			l = 0
			while len(data) < datalen:
				data += self._readline()
			if len(data) != datalen:
				self.notice("not not read the number of byte expected. read %d expected %d" % (len(data),datalen),1)
				return False
		except:
			return False
		
		data = data.strip()
		
		try:
			footer = self._readline()
			if footer[0] != 'C':
				self.notice("Failure in server answer")
				return False
		except:
			self.notice("Failure in server answer")
			return False
		
		return data
	
	def set_sources (self,sources):
		self.notice('setting source ' + source,2)
		self._send('!s-%s\n' % sources)
		
		result = self._response()
		
		if result == False or result == True:
			return False
		
		return result.split(' ')
	
	def get_sources(self):
		result = self._cache_query('source',source,2)
		if result != False:
			return result
		
		self.notice('getting sources ' + source,2)
		self._send('!s-lc\n')
		
		result = self.response()
		
		if result == False or result == True:
			return False
		
		self._cache_add('source',source,result)
		return result
	
	def get_routes_by_origin (self,origin):
		result = self._cache_query('origin',origin)
		if result != False:
			return result
		
		self._send('!g%s\n' % origin)
		
		result = self._response()
		if result == False or result == True:
			return False

		result = self._classful_fix(result.split(' '))
		result.sort()

		self._cache_add('origin',origin,result)
		
		return result
	
	def get_data_by_set (self,aset):
		result = self._cache_query('set',aset)
		if result != False:
			return result
		
		routes = []
		
		self._send('!i%s,1\n' % aset)
		
		result = self._response()
		if result == False or result == True:
			return False
		
		autnumlist = list(set(result.split(' '))) # set are unique
		
		if result.find('.') < 0:
			for as_set in autnumlist:
				result = self.get_routes_by_origin(as_set)
				
				if result == False or result == True:
					continue
				
				routes.extend(result)
		else:
			# as-set can't contain ., must be route-set
			routes = self._classful_fix(autnumlist)
			# XXX: We do a = and not a merge is it correct.
		
		routes.sort()
		routes = list(set(routes))

		back = {}
		back['routes'] = routes
		back['asns'] = autnumlist
		
		return back
	
	def get_member_by_set (self,aset,recursive):
		routes = []
		
		if recursive == True:
			self._send('!i%s,1\n' % aset)
		else:
			self._send('!i%s,n\n' % aset)
		
		result = self._response()
		if result == False or result == True:
			return False
		
		data = set(result.split(' '))
		
		return data
	
	def _classful_fix (self,routes):
		for route in routes:
			if route.find('/') < 0:
				try:
					first = int(route.split('.')[0])
				except:
					self.notice('invalid prefix : %s' % route,1)
					routes.remove(route)
					continue

				if first > 0 and first < 128:
					lenght = '/8'
				elif first > 0 and first < 192:
					lenght = '/16'
				elif first > 0 and first < 255:
					lenght = '/24'
				# Does not really check for multicast ?
				else:
					self.notice('invalid prefix : %s' % route,1)
					routes.remove(route)
					continue
				routes.remove(route)
				routes.append(route+lenght)
		return routes
	
	def set_cache(self,caching):
		if caching:
			self.caching = True
		else:
			caching = False
	
	def clear_cache (self):
		self.cache = {}
	
	def _cache_query(self,atype,record):
		if self.caching:
			try:
				self.cache[atype][record]['hit'] += 1
				return self.cache[atype][record]['data']
			except:
				pass
		return False
	
	def _cache_add (self,atype,record,data):
		if self.caching:
			try:
				self.cache[atype][record]['data'] = data
			except:
				if not self.cache.has_key(atype):
					self.cache[atype] = {}
				if not self.cache[atype].has_key(record):
					self.cache[atypee][record] = {}

			self.cache[atype][record]['time'] = time.time() # time.ctime() for human readeable
			self.cache[atype][record]['hit'] = 0
	
	def set_debug (self,debug):
		self.debug = debug
	
	def notice(self,msg,level=0):
		if level <= self.debug:
			print >> sys.stderr,"irr client :",msg

if __name__ == '__main__':
	import os
	import sys
	parent = os.path.abspath(os.path.join(os.path.abspath(os.path.dirname(__file__)),'..'))
	sys.path.append(parent)
	from conf.param import option
	from conf.irrpt import cfg
	
	asn = []
	
	for a in option.get('cmdline',[]):
		if a[:2] == 'AS':
			b = a[2:]
			try: asn.append(('asn',str(int(b))))
			except: asn.append(('as-set',a))
		else:
			asn.append(('as-set',a))
	
	if asn == []:
		from conf.irrdb import cfg as config
		for c in config:
			asn.append(c[1])
	
	server = cfg['fetch']['host']
	port = cfg['fetch']['port']
	source = cfg['fetch']['sources']
	debug = 2
	
	irr = IRR(server,port)
	irr.set_debug(debug)
	if source != 'all':
		irr.set_sources(source)
	
	routes = []
	for a in asn:
		if a[0] == 'asn':
			routes = irr.get_routes_by_origin(a[1])
		if a[0] == 'as-set' or routes == False:
			data = irr.get_data_by_set(a[1])
			if data:
				routes = data['routes']
				anslist = data['asns']
		else:
			asnlist = asn
	
	irr.close()

	for route in routes:
		print route
	
	sys.exit(0)
	



