#!/usr/bin/env python
import os
import sys
parent = os.path.abspath(os.path.join(os.path.abspath(os.path.dirname(__file__)),'..'))
sys.path.append(parent)

import ip
import cidr
from copy import copy


class Aggregate:
	def __init__ (self):
		self.range = dict()
		self.added = dict()
		self.aggregated = True
	
	def append (self,net,size):
		self.aggregated = False
		next = net + size
		# optimisation needed by the _aggregate algorithm.
		if self.added.has_key(net):
			next = max(next,self.added[net])
		self.added[net] = next

	def _aggregate (self):
		# Do not do the aggregation if there is no changes
		if self.aggregated:
			return
		
		# Take our current known route and place them as tuple in a orderd list
		keys = self.range.keys()
		keys.sort()
		working = [(s,self.range[s]) for s in keys]
		
		# Take our new routes and do the same
		keys = self.added.keys()
		keys.sort()
		adding = [(s,self.added[s]) for s in keys]
		
		# The tuples are expressed as a tuple (start ip, start ip next range)
		# and NOT (start ip, end ip)
		
		if len(adding) == 0:
			self.aggregated = True
			return
		
		if len(working) == 0:
			working.append(adding[0])
			adding = adding[1:]
		
		debug = 0
		if debug: print "working before is ", [(ip.toip(s), ip.toip(e)) for (s,e) in working]
		if debug: print "adding  before is ", [(ip.toip(s), ip.toip(e)) for (s,e) in adding]
		
		# for all the range we need to add to the currents
		for new_s,new_e in adding:
			last = len(working) - 1
			last_s,last_e = working[last]
			
			if debug: print
			if debug: print "working", [(ip.toip(s), ip.toip(e)) for (s,e) in working]
			if debug: print "new    ", ip.toip(new_s), ip.toip(new_e)
			
			# if the start of our range is bigger than the end of the last one, append
			if new_s > last_e:
				working.append((new_s,new_e))
				if debug: print "Append"
				continue
			
			# for all the entries in the current range, reversed
			i_first = -1
			i_last = -1
			
			for i in range(last,-1,-1):
				work_s,work_e = working[i]
				if debug: print "process", ip.toip(work_s), ip.toip(work_e)
				# find the index of the first entry which start is smaller or equal than our last
				# find the index of the firsts entry which end is bigger or equal than our first
				if new_s <= work_e:
					i_first = i
				if new_e >= work_s:
					i_last = i
					if debug: print 'break'
					break
			
			if debug: print "first", i_first, '   ', ip.toip(working[i_first][0]),ip.toip(working[i_first][1])
			if debug: print "fast", i_last, '    ', ip.toip(working[i_last][0]),ip.toip(working[i_last][1])

			# we went all the way and found nothing, 
			if i_last == -1 or i_first == -1:
				# I do not believe it is possible ... but, hey, better catch it
				raise
			
			# we take the smallest of the firsts and the highest of the second
			# and replace the middle
			w_s = working[i_first][0]
			w_e = working[i_last][1]
			if debug: print "working found", ip.toip(w_s), ip.toip(w_e)
			start = min(new_s,w_s)
			end = max(new_e,w_e)

			if i_first == i_last:
				working[i_first] = (start,end)
			else:
				working[i_first:i_last] = (start,end)
		
		self.range = {}
		for (start,end) in working:
			self.range[start] = end
		
		self.aggregated = True

	def __iter__ (self):
		self._aggregate()
		keys = self.range.keys()
		keys.sort()
		for start in keys:
			end = self.range[start]
			size = end - start
			ranges = ip.decompose_range(start,size)
			for r in ranges:
				yield cidr.CIDR(r[0],ip.tobits(r[1]))

	def slice (self,net,size):
		next = net + size

		for start in self.range.keys():
			end = self.range[start]

			if net == start and next == end:
				del self.range[start]
				return True
			if net >= start and next <= end:
				del self.range[start]
				if net > start:
					self._append(start,net)
				if end > next:
					self._append(next,end)
				return True
		return False

	def __str__ (self):
		res = ""
		for cidr in self:
			res += str(cidr) + " \n"
		return res[:-1]

if __name__ == '__main__':
	from conf.param import option

	def usage (problem):
		print >> sys.stderr , 'usage: %s [upto] [smallest]' % sys.argv[0]
		print >> sys.stderr , '          upto: the longest prefix you want in your aggregated result'
		print >> sys.stderr , '          smallest: the longer prefix you will accept to perform aggregation calculation on'
		print >> sys.stderr , '          -a, --aligned: only range which are correctly aligned for their size'
		print >> sys.stderr , problem
		sys.exit(1)

	# get the parameters and set them to /24 otherwise
	cmdline = option.get('cmdline',['/24'])
	cmdline.append('/32')
	aligned = option.get('aligned',False)
	
	try:
		upto = int(cmdline[0].replace('/',''))
		smallest = int(cmdline[1].replace('/',''))
		
		if upto < 0 or upto > 32:
			raise
		if smallest < 0 or upto > 32:
			raise
	except:
		usage('invalid parameters')

	agg = Aggregate()
	for line in sys.stdin.readlines():
		try:
			net,mask = line.strip().split('/')
		except:
			striped = line.strip()
			if striped != '' and striped[0] != '#':
				print >> sys.stderr, 'invalid entry %s' % line.strip()
			continue
		
		try:
			network = ip.todecimal(net)
		except:
			if ip.isunsanitisedip(net):
				network = ip.sanitise(net)
				network = ip.todecimal(network)
			else:
				print >> sys.stderr, 'invalid entry %s' % line.strip()
				continue

		try:
			mask = int(mask)
			if mask < 0 or mask > 32:
				raise
		except:
			print >> sys.stderr, 'invalid entry %s' % line.strip()
			continue
		
		size = ip.tosize(mask)

		if aligned:
			if not ip.aligned(network,size):
				print >> sys.stderr, 'excluding %s/%s , not aligned' % (net,mask)
		elif mask > smallest:
			print >> sys.stderr, 'excluding %s/%s , too small' % (net,mask)
		else:
			agg.append(network,size)
	
	for r in agg:
		if r.bits <= upto:
			print r
	sys.exit(0)

