#!/usr/bin/env python

# All the function expect the IP (strings) to be of the form a.b.c.d

from struct import pack, unpack
from socket import htonl, ntohl, inet_aton
import re

_sizetoslash = dict()
_slashtosize = dict()

for bits in xrange(1,32+1):
	size = pow(2,32-bits)
	_sizetoslash[size] = bits
	_slashtosize[bits] = size

# return the number of ip contained in a /bits
# input : integer : bits : size of the /
# output : integer : number of ip in that netmask
def tosize(bits):
	return _slashtosize[int(bits)]

# return the netmask in / format for a given number of ip
# input : integer : size : number of ip 
# output : integer : the number of bit for the / of the netmask
def tobits(size):
	return _sizetoslash[int(size)]

# Make sure an ip is of the form a.b.c.d 
# (and not any of its shortcut a.b or a.b.c
# input : string : an ip 
# output : string : an ip
def sanitise (ips):
	b = ips.split('.')
	l = len(b)
	if l == 2: b.extend(['0','0'])
	if l == 3: b.extend(['0'])
	return '.'.join(b)

# is the data an ip of the form a,b,c,d
# input : string : an ip 
# output : boolean : answer
ipchecker = re.compile('^(\d{3}\.){1,3}\d{1,3}$')
def isip(value):
	if ipchecker.match(value):
		return True
	return False

# variant :
# an ip can be a.b.c.d , the common form
# a.b.c which is a shortcut for a.b.c.0
# a.b which is a shortcut for a.b.0.0
ipchecker = re.compile('^(\d{1,3}\.){1,3}\d{1,3}$')
def isunsanitisedip(value):
	if ipchecker.match(value):
		return True
	return False

# Convert a numeric IP to its A.B.C.D representation
# Input : integer : value : an ip 
# Output : string : the same ip 
def toip(value):
	value = htonl(value)
	ip = pack('I', value)
	a, b, c, d = unpack('BBBB', ip)
	return '.'.join([str(i) for i in a, b, c, d])

#def toip(value):
#	d = value & 255
#	c = (value >> 8) & 255
#	b = (value >> 16) & 255
#	b = (value >> 24) & 255
#	return '.'.join([a,b,c,d])

# Convert a A.B.C.D IP to its numeric value
# Input : string : value : an ip under the form a.b.c.d
# Output : integer : the ip under integer form
#def todecimal(value):
#	address = inet_aton(value)
#	unpacked = ntohl(unpack('I', address)[0])
#	packed = pack('I', unpacked)
#	ordered = unpack('I', packed)[0]
#	return ordered

def todecimal(value):
	a,b,c,d = [int(i) for i in value.split('.')]
	return (a<<24) + (b<<16) + (c<<8) + d

# Reverse an ip address for inclusion in an arpa entry
# Input : string : value : an ip under the form a.b.c.d
# Outpout : string : a string d.c.b.a
def iptoarpa (ip):
	a,b,c,d = ip.split('.')
	return '.'.join([d, c, b, a])

# XXX: The name of this function really s****
# It answers : Is this number is sum of 2^x number
# Input : integet : size : an ip
# Ouput : boolean : is the ip one of the start of the 32 classless range
def israngesize(size):
	return _sizetoslash.has_key(size)

# Can a network start at this ip and be of that size
# Input : integer : network : an IP we want to check if it start a range of lenght size
# Input : integer : size : the size of the network expressed in number of ips
# Outpout : boolean : is the ip of the network the start of a range.
def aligned(network,size):
	return (int(network/size))*size == network

# Decompose a range 328 into the sum of 256 + 64 + 8
# Input : integer : size, the network size to decompose
# Output : list of integer : answer ordered bigger to smaller
def decompose_size(size):
	start = 1
	result = list()
	loop = True
	while loop:
		for bits in xrange(start,32+1):
			s = tosize(bits)
			if s <= size:
				result.append(s)
				size -= s
				start = bits
				if size == 0:
					loop = False
					break
		if loop:
			raise "Can't decompose size ", size
	return result

# Decompose consecutive ranges into their biggest group
# WARNING: This function expect the size ordered bigger to smaller
# Input : integer : network : a range start
# Input : integer : size : a range lenght
def decompose_range(network,size):
	# Search for the biggest component of the range
	# if the range is 328 (256 + 64 + 8) we look for 256
	# and store it in boundry
	for bits in xrange(32,-1,-1):
		s = tosize(bits)
		if s > size:
			boundry = tosize(bits+1)
			break

	# Look for the ip of ip starting or ending this range
	limit = ((network/boundry)+1)*boundry

	# recursively split the two new ranges until we can not find any way to break them
	if size > 1 and limit > network and limit < network + size:
		parta = decompose_range(network,limit-network)
		partb = decompose_range(limit,size-limit+network)
		result = parta
		result.extend(partb)
		return result

	# we want to find how many range their is.
	result = list()
	subsize = decompose_size(size)
	nb_ranges = len(subsize)
	# if this is a single valid range (ie a valid /something), return it
	if nb_ranges == 1:
		result.append([network,subsize[0]])
		return result
	
	# Find the biggest netmask possible for the range start we have just found
	while nb_ranges:
		# size in the list are ordered bigger to smaller
		# You must not modify an object you are iterating over, hence the copy via slice
		for s in subsize[:]:
			if aligned(network,s):
				subsize.remove(s)
				result.append([network,s])
				network += s
				break
		# is the test necessary ?? was surely here to catch some bug but now ?
		if nb_ranges != len(subsize):
			nb_ranges = len(subsize)
		else:
			raise "Can not get here", str(toip(network)) + " " + str(size) + " " + str(subsize)
	return result
